@@ -23,20 +23,22 @@
 #include "plotitem.h"
 
 #include "analitza/analitzautils.h"
+#include <analitza/variables.h>
+#include <analitzaplot/functiongraph.h>
 
 #include <QDebug>
 #include <QIcon>
 #include <QPixmap>
 #include <QCoreApplication>
-#include <analitzaplot/functiongraph.h>
 
 using namespace Analitza;
 
-Q_DECLARE_METATYPE(PlotItem*);
+Q_DECLARE_METATYPE(PlotItem*)
 
 PlotsModel::PlotsModel(QObject* parent)
     : QAbstractListModel(parent)
     , m_resolution(500)
+    , m_namingCount(0)
 {}
 
 PlotsModel::~PlotsModel()
@@ -44,6 +46,11 @@ PlotsModel::~PlotsModel()
     clear();
 }
 
+QHash<int, QByteArray> PlotsModel::roleNames() const
+{
+    return QAbstractListModel::roleNames().unite({{ DescriptionRole, "description" }});
+}
+
 void PlotsModel::clear()
 {
     if (!m_items.isEmpty()) {
@@ -67,8 +74,8 @@ QVariant PlotsModel::headerData(int section, Qt::Orientation orientation, int ro
     if(role==Qt::DisplayRole && orientation==Qt::Horizontal) {
         switch(section) 
         {
-			case 0: return QCoreApplication::translate("@title:column", "Name");
-			case 1: return QCoreApplication::translate("@title:column", "Plot");
+            case 0: return QCoreApplication::translate("@title:column", "Name");
+            case 1: return QCoreApplication::translate("@title:column", "Plot");
         }
     }
     
@@ -112,6 +119,8 @@ QVariant PlotsModel::data(const QModelIndex & index, int role) const
             return int(tmpcurve->spaceDimension());
         case PlotRole:
             return qVariantFromValue<PlotItem*>(tmpcurve);
+        case DescriptionRole:
+            return tmpcurve->display();
     }
 
     
@@ -216,12 +225,12 @@ void PlotsModel::addPlot(PlotItem* it)
 
 void PlotsModel::updatePlot(int row, PlotItem* it)
 {
-	it->setModel(this);
-	delete m_items[row];
-	m_items[row]=it;
-	
-	QModelIndex idx = index(row);
-	emit dataChanged(idx, idx);
+    it->setModel(this);
+    delete m_items[row];
+    m_items[row]=it;
+    
+    QModelIndex idx = index(row);
+    emit dataChanged(idx, idx);
 }
 
 void PlotsModel::emitChanged(PlotItem* it)
@@ -233,18 +242,18 @@ void PlotsModel::emitChanged(PlotItem* it)
 
 QModelIndex PlotsModel::indexForName(const QString& name)
 {
-	const int rows = rowCount();
-	for(int i=0; i<rows; i++) {
-		QModelIndex idx = index(i);
-		if(idx.data().toString()==name)
-			return idx;
-	}
-	return QModelIndex();
+    const int rows = rowCount();
+    for(int i=0; i<rows; i++) {
+        QModelIndex idx = index(i);
+        if(idx.data().toString()==name)
+            return idx;
+    }
+    return QModelIndex();
 }
 
 QString PlotsModel::freeId() const
 {
-	return "f"+QString::number(m_namingCount);
+    return "f"+QString::number(m_namingCount);
 }
 
 void PlotsModel::setResolution(int res)
@@ -257,3 +266,30 @@ void PlotsModel::setResolution(int res)
     }
 }
 
+static QColor randomFunctionColor() { return QColor::fromHsv(qrand()%255, 255, 225); }
+
+QStringList PlotsModel::addFunction(const QString& expression, Dimension dim, Analitza::Variables* vars)
+{
+    Analitza::Expression e(expression, Analitza::Expression::isMathML(expression));
+
+    QString fname;
+    do {
+        fname = freeId();
+    } while(vars && vars->contains(fname));
+    QColor fcolor = randomFunctionColor();
+
+    QStringList err;
+    PlotBuilder req = PlotsFactory::self()->requestPlot(e, dim, vars);
+    if(req.canDraw()) {
+        auto it = req.create(fcolor, fname);
+
+        if(it->isCorrect())
+            addPlot(it);
+        else {
+            err = it->errors();
+            delete it;
+        }
+    }
+
+    return err;
+}
